const degit = require("degit");
const path = require("path");
const fs = require("fs");
const inquirer = require("inquirer");

async function createProject(projectName, skipQuestions) {
  const repo = "mativiaggio/menth-template";
  const emitter = degit(repo, {
    cache: false,
    force: true,
    verbose: true,
  });

  console.log(`Creating new project: ${projectName}`);

  emitter.on("info", (info) => {
    console.log(info.message);
  });

  await emitter.clone(path.join(process.cwd(), projectName));

  console.log("Project created successfully!");
  console.log("Installing dependencies...");

  process.chdir(projectName);
  require("child_process").execSync("npm install", { stdio: "inherit" });

  console.log("Dependencies installed successfully!");
  console.log(`Run "cd ${projectName} && npm run dev" to start the development server.`);

  await askEnvironmentVariables(skipQuestions);
}

async function askEnvironmentVariables(skipQuestions) {
  const defaultValues = {
    PORT: "3000",
    MONGO_PASSWORD: "",
    MONGO_URI: "",
    EMAIL_SERVICE: "gmail",
    EMAIL_PORT: "587",
    EMAIL_USER: "",
    EMAIL_PASSWORD: "",
    NODE_ENV: "development",
    HOST: "http://localhost:3000",
  };

  if (skipQuestions) {
    writeEnvFile(defaultValues);
    return;
  }

  const questions = [
    {
      type: "input",
      name: "PORT",
      message: "Enter the port number:",
      default: defaultValues.PORT,
    },
    {
      type: "input",
      name: "MONGO_PASSWORD",
      message: "Enter your MongoDB password:",
      default: defaultValues.MONGO_PASSWORD,
    },
    {
      type: "input",
      name: "MONGO_URI",
      message: "Enter your MongoDB URI:",
      default: defaultValues.MONGO_URI,
    },
    {
      type: "input",
      name: "EMAIL_SERVICE",
      message: "Enter your email service (e.g., gmail):",
      default: defaultValues.EMAIL_SERVICE,
    },
    {
      type: "input",
      name: "EMAIL_PORT",
      message: "Enter your email port:",
      default: defaultValues.EMAIL_PORT,
    },
    {
      type: "input",
      name: "EMAIL_USER",
      message: "Enter your email user:",
      default: defaultValues.EMAIL_USER,
    },
    {
      type: "password",
      name: "EMAIL_PASSWORD",
      message: "Enter your email password:",
      mask: "*",
      default: defaultValues.EMAIL_PASSWORD,
    },
    {
      type: "input",
      name: "NODE_ENV",
      message: "Enter your node environment:",
      default: defaultValues.NODE_ENV,
    },
    {
      type: "input",
      name: "HOST",
      message: "Enter your host:",
      default: defaultValues.HOST,
    },
  ];

  const answers = await inquirer.prompt(questions);
  writeEnvFile(answers);
}

function writeEnvFile(envValues) {
  const envContent = Object.keys(envValues)
    .map((key) => `${key}=${envValues[key]}`)
    .join("\n");

  fs.writeFileSync(path.join(process.cwd(), ".env"), envContent);

  console.log(".env file created successfully!");
}

const projectName = process.argv[2];
const skipQuestions = process.argv.includes("-y");

if (!projectName) {
  console.error("Please specify the project directory:");
  console.error("  npx create-menth-app <project-directory> [-y]");
  process.exit(1);
}

createProject(projectName, skipQuestions);
